import subprocess
from argparse import ArgumentParser
from rkd.api.contract import ExecutionContext
from ...formatting import development_formatting
from .base import BaseDeploymentTask


class ManageVagrantTask(BaseDeploymentTask):
    """Controls a test virtual machine using Vagrant"""

    def get_group_name(self) -> str:
        return ':harbor:deployment'

    def get_name(self) -> str:
        return ':vagrant'

    def format_task_name(self, name) -> str:
        return development_formatting(name)

    def configure_argparse(self, parser: ArgumentParser):
        parser.add_argument('--cmd', '-c', required=True, help='Vagrant commandline')

    def run(self, context: ExecutionContext) -> bool:
        cmd = context.get_arg('--cmd')

        try:
            subprocess.check_call('cd %s && vagrant %s' % (self.ansible_dir, cmd), shell=True)

        except subprocess.CalledProcessError:
            return False

        return True




import os
import subprocess
from io import StringIO
from rkd.api.inputoutput import BufferedSystemIO
from rkd.api.inputoutput import IO
from rkd.api.contract import ExecutionContext
from rkd.api.syntax import TaskDeclaration
from rkd_harbor.test import BaseHarborTestClass
from rkd_harbor.test import TestTask


class TestHooksFeature(BaseHarborTestClass):
    def _prepare_test_data(self):
        with open(self.get_test_env_subdirectory('hooks.d/pre-upgrade') + '/whoami.sh', 'w') as fw:
            fw.write('''
                #!/bin/bash
                echo " >> This is a whoami.sh hook, test: $(whoami)"
            ''')

        with open(self.get_test_env_subdirectory('hooks.d/post-upgrade') + '/history.sh', 'w') as fw:
            fw.write('''
                #!/bin/bash
                echo "25 June 1978 the rainbow flag was first flown at the San Francisco Gay Freedom Day Parade. The flag was designed by artist Gilbert Baker, and hand-dyed and stitched by 30 volunteers."
            ''')

        subprocess.check_call(['chmod', '+x', self.get_test_env_subdirectory('hooks.d/pre-upgrade') + '/whoami.sh'])
        subprocess.check_call(['chmod', '+x', self.get_test_env_subdirectory('hooks.d/post-upgrade') + '/history.sh'])

    def test_functional_hooks_are_executed_when_exists_and_files_with_extension_only_are_skipped(self):
        """Given we have an example hooks in pre-upgrade/whoami.sh and in post-upgrade/history.sh
        And we try to run those hooks using hooks_executed()
        Then we will see output produced by those scripts
        And .dotfiles will be ignored
        """

        self._prepare_test_data()

        buffer = StringIO()
        hooks_capturing_io = IO()

        task = TestTask()
        task._io = BufferedSystemIO()
        ctx = ExecutionContext(
            TaskDeclaration(task),
            args={},
            env={}
        )

        with hooks_capturing_io.capture_descriptors(stream=buffer, enable_standard_out=True):
            with task.hooks_executed(ctx, 'upgrade'):
                pass

        self.assertIn('>> This is a whoami.sh hook, test:', buffer.getvalue(),
                      msg='Expected pre-upgrade hook to be ran')

        self.assertIn('25 June 1978 the rainbow flag was first flown', buffer.getvalue(),
                      msg='Expected post-upgrade hook to be ran')

        self.assertIn('pre-upgrade/whoami.sh', task._io.get_value())
        self.assertNotIn('.gitkeep', task._io.get_value())

    def test_functional_execute_hooks_executes_post_upgrade_hooks(self):
        """Assert that post-upgrade/history.sh is executed"""

        self._prepare_test_data()

        buffer = StringIO()
        hooks_capturing_io = IO()

        task = TestTask()
        task._io = BufferedSystemIO()
        ctx = ExecutionContext(
            TaskDeclaration(task),
            args={},
            env={}
        )

        with hooks_capturing_io.capture_descriptors(stream=buffer, enable_standard_out=True):
            task.execute_hooks(ctx, 'post-upgrade')

        self.assertIn('25 June 1978 the rainbow flag was first flown', buffer.getvalue(),
                      msg='Expected post-upgrade hook to be ran')

    def test_non_existing_dir_is_skipped(self):
        """Assert that non-existing directory does not cause exception, but will be skipped"""

        task = TestTask()
        task._io = BufferedSystemIO()
        task._io.set_log_level('debug')
        ctx = ExecutionContext(
            TaskDeclaration(task),
            args={},
            env={}
        )

        task.execute_hooks(ctx, 'non-existing-directory')
        self.assertIn('Hooks dir "./hooks.d//non-existing-directory/" not present, skipping', task._io.get_value())




import os
import subprocess
from abc import ABC
from jinja2 import Environment
from jinja2 import FileSystemLoader
from jinja2 import StrictUndefined
from jinja2.exceptions import UndefinedError
from argparse import ArgumentParser
from rkd.api.contract import ExecutionContext
from rkd.yaml_parser import YamlFileLoader
from rkd.exception import MissingInputException
from rkd.api.inputoutput import Wizard
from ..base import HarborBaseTask
from ...exception import MissingDeploymentConfigurationError

HARBOR_ROOT = os.path.dirname(os.path.realpath(__file__)) + '/../../deployment/files'


class BaseDeploymentTask(HarborBaseTask, ABC):
    ansible_dir: str = '.rkd/deployment'
    _config: dict
    vault_args: list = []

    def get_config(self) -> dict:
        """Loads and parses deployment.yml file.

        Supports:
            - Ansible Vault encryption of deployment.yml
            - SSH private key storage inside deployment.yml
        """

        deployment_filenames = ['deployment.yml', 'deployment.yaml']

        try:
            self._config
        except AttributeError:

            # try multiple files
            for filename in deployment_filenames:
                if os.path.isfile(filename):

                    #
                    # Check file contents before
                    #
                    with open(filename, 'rb') as f:
                        content = f.read().decode('utf-8')
                        #
                        # When file is encrypted, then decrypt it
                        #
                        if content.startswith('$ANSIBLE_VAULT;'):
                            tmp_vault_path, tmp_vault_filename = self.temp.create_tmp_file_path()

                            self.io().info('Decrypting deployment file')
                            self.sh('cp %s %s' % (filename, tmp_vault_path))

                            self.io().info_msg('Need a vault passphrase to decrypt "%s"' % filename)
                            self.rkd([':harbor:vault:encrypt', '-d', tmp_vault_path] + self.vault_args)
                            self._config = YamlFileLoader(self._ctx.directories).load_from_file(
                                tmp_vault_filename,
                                'org.riotkit.harbor/deployment/v1'
                            )

                            self._process_config_private_keys()
                            return self._config

                    self._config = YamlFileLoader(self._ctx.directories).load_from_file(
                        filename,
                        'org.riotkit.harbor/deployment/v1'
                    )

                    self._process_config_private_keys()
                    return self._config

            raise MissingDeploymentConfigurationError()

        return self._config

    def _process_config_private_keys(self):
        """Allow private keys to be pasted directly to the deployment.yml

        On-the-fly those keys will be written into the temporary directory
        """

        for group_name, nodes in self._config['nodes'].items():
            for node_num in range(0, len(nodes)):
                if 'private_key' not in self._config['nodes'][group_name][node_num]:
                    continue

                if '-----BEGIN' not in self._config['nodes'][group_name][node_num]['private_key']:
                    continue

                tmp_path = self.temp.assign_temporary_file(mode=0o700)

                self.io().info('Storing inline private key as "%s"' % tmp_path)
                with open(tmp_path, 'w') as key_file:
                    key_file.write(self._config['nodes'][group_name][node_num]['private_key'].strip())
                    key_file.write("\n")

                self._config['nodes'][group_name][node_num]['private_key'] = tmp_path

    def _verify_synced_version(self, abs_ansible_dir: str):
        """Verifies last synchronization - displays warning if Harbor version was changed after last
        files synchronization"""

        if not os.path.isfile(abs_ansible_dir + '/.synced'):
            return

        with open(abs_ansible_dir + '/.synced', 'rb') as f:
            synced_version = f.read().decode('utf-8').strip()
            actual_version = self.get_harbor_version()

            if synced_version != actual_version:
                self.io().warn('Ansible deployment in .rkd/deployment is not up-to-date. We recommend to update' +
                               ' from %s to %s' % (synced_version, actual_version))

    def _write_synced_version(self, abs_ansible_dir: str):
        """Writes information about, in which Harbor version the files were synced last time"""

        with open(abs_ansible_dir + '/.synced', 'wb') as f:
            f.write(self.get_harbor_version().encode('utf-8'))

    def role_is_installed_and_configured(self) -> bool:
        return os.path.isfile(self.ansible_dir + '/.synced')

    def _ask_and_set_var(self, ctx: ExecutionContext, arg_name: str, title: str, attribute: str, secret: bool):
        """Ask user an interactive question, then add answer to the deployment.yml loaded in memory

        The variable will be appended to any node, where the variable is empty.
        Example: We have 5 servers, 3 without a password. So the password will be applied to 3 servers.
        """

        self.get_config()

        if not ctx.get_arg(arg_name):
            return

        wizard = Wizard(self).ask(title, attribute=attribute, secret=secret)

        for group_name, nodes in self._config['nodes'].items():
            node_num = 0

            for node in nodes:
                node_num += 1
                if attribute in self._config['nodes'][group_name][node_num - 1]:
                    continue

                self._config['nodes'][group_name][node_num - 1][attribute] = wizard.answers[attribute]

    def install_and_configure_role(self, ctx: ExecutionContext, force_update: bool = False) -> bool:
        """Install an Ansible role from galaxy, and configure playbook, inventory, all the needed things"""

        abs_ansible_dir = os.path.realpath(self.ansible_dir)
        should_update = force_update or not os.path.isfile(abs_ansible_dir + '/.synced')

        self.io().info('Checking role installation...')
        self._silent_mkdir(abs_ansible_dir)
        self._verify_synced_version(abs_ansible_dir)

        # optionally ask user and set facts such as passwords, key paths, sudo passwords
        # ansible-vault password prompt is handed by ansible-vault itself
        self._ask_and_set_var(ctx, '--ask-ssh-login', 'SSH username', 'user', secret=True)
        self._ask_and_set_var(ctx, '--ask-ssh-pass', 'SSH password', 'password', secret=True)
        self._ask_and_set_var(ctx, '--ask-ssh-key-path', 'SSH private key path', 'private_key', secret=False)
        self._ask_and_set_var(ctx, '--ask-sudo-pass', 'Sudo password for remote machines', 'sudo_pass', secret=True)

        if not self._synchronize_structure_from_template(abs_ansible_dir, only_jinja_templates=True):
            self.io().error_msg('Cannot synchronize templates')
            return False

        if should_update:
            self.io().info('Role will be updated')

            if not self._synchronize_structure_from_template(abs_ansible_dir):
                self.io().error_msg('Cannot synchronize structure')
                return False

            self.io().debug('Downloading fresh role...')
            self.download_roles()

            self._write_synced_version(abs_ansible_dir)

        return True

    def download_roles(self):
        self.sh(' '.join([
            'ansible-galaxy',
            'install', '-r', self.ansible_dir + '/requirements.yml',
            '-p', self.ansible_dir + '/roles/',
            '--force'
        ]), capture=False)

    def _synchronize_structure_from_template(self, abs_ansible_dir: str, only_jinja_templates: bool = False) -> bool:
        """Synchronizes template structure into .rkd/deployment"""

        self.io().debug(
            'Synchronizing structure from template (only_jinja_templates=' + str(only_jinja_templates) + ')')

        # synchronize directory structure
        for root, subdirs, files in os.walk(HARBOR_ROOT):
            relative_root = root[len(HARBOR_ROOT) + 1:]

            self._silent_mkdir(abs_ansible_dir + '/' + relative_root)

            for file in files:
                if only_jinja_templates and not file.endswith('.j2'):
                    continue

                abs_src_file_path = root + '/' + file
                abs_dest_file_path = abs_ansible_dir + '/' + relative_root + '/' + file

                if not self._copy_file(abs_src_file_path, abs_dest_file_path):
                    self.io().error('Cannot process file %s' % abs_dest_file_path)
                    return False

        return True

    def _copy_file(self, abs_src_file_path: str, abs_dest_file_path: str):
        """Copies a file from template directory - supports jinja2 files rendering on-the-fly"""

        if abs_dest_file_path.endswith('.j2'):
            abs_dest_file_path = abs_dest_file_path[:-3]

            with open(abs_src_file_path, 'rb') as f:
                tpl = Environment(loader=FileSystemLoader(['./', './rkd/deployment']), undefined=StrictUndefined)\
                        .from_string(f.read().decode('utf-8'))

            try:
                variables = self._prepare_variables()

                with open(abs_dest_file_path, 'wb') as f:
                    f.write(tpl.render(**variables).encode('utf-8'))

            except UndefinedError as e:
                self.io().error(str(e) + " - required in " + abs_src_file_path + ", please define it in deployment.yml")
                return False

            return True

        subprocess.check_call(['cp', '-p', abs_src_file_path, abs_dest_file_path])
        self.io().debug('Created ' + abs_dest_file_path)
        return True

    def _prepare_variables(self):
        """Glues together variables from environment and from deployment.yaml for exposing in JINJA2 templates"""

        variables = {}
        variables.update(os.environ)
        variables.update(self.get_config())

        if 'git_url' not in variables:
            variables['git_url'] = subprocess\
                .check_output(['git', 'config', '--get', 'remote.origin.url']).decode('utf-8')\
                .replace('\n', '')\
                .strip()

        if 'git_secret_url' not in variables:
            variables['git_secret_url'] = variables['git_url'].replace('\n', '')

        return variables

    def _preserve_vault_parameters_for_usage_in_inner_tasks(self, ctx: ExecutionContext):
        """Preserve original parameters related to Vault, so those parameters can be propagated to inner tasks"""

        try:
            vault_passwords = ctx.get_arg_or_env('--vault-passwords')
        except MissingInputException:
            vault_passwords = ''

        # keep the vault arguments for decryption of deployment.yml
        self.vault_args = ['--vault-passwords=' + vault_passwords]
        if ctx.get_arg('--ask-vault-pass'):
            self.vault_args.append('--ask-vault-pass')

    def _get_vault_opts(self, ctx: ExecutionContext, chdir: str = '') -> str:
        """Creates options to pass in Ansible Vault commandline

        The output will be a temporary vault file with password entered inline or a --ask-vault-pass switch
        """

        try:
            vault_passwords = ctx.get_arg_or_env('--vault-passwords').split('||')
        except MissingInputException:
            vault_passwords = []

        num = 0
        opts = ''
        enforce_ask_pass = ctx.get_arg('--ask-vault-pass')

        for passwd in vault_passwords:
            num = num + 1

            if not passwd:
                continue

            if passwd.startswith('./') or passwd.startswith('/'):
                if os.path.isfile(passwd):
                    opts += ' --vault-password-file="%s" ' % (chdir + passwd)
                else:
                    self.io().error('Vault password file "%s" does not exist, calling --ask-vault-pass' % passwd)
                    enforce_ask_pass = True
            else:
                tmp_vault_file = self.temp.assign_temporary_file(mode=0o644)

                with open(tmp_vault_file, 'w') as f:
                    f.write(passwd)

                opts += ' --vault-password-file="%s" ' % (chdir + tmp_vault_file)

        if enforce_ask_pass:
            opts += ' --ask-vault-pass '

        return opts

    @classmethod
    def _add_vault_arguments_to_argparse(cls, parser: ArgumentParser):
        parser.add_argument('--ask-vault-pass', '-v', help='Ask for vault password interactively', action='store_true')
        parser.add_argument('--vault-passwords', '-V', help='Vault passwords separated by "||" eg. 123||456')

    @classmethod
    def _add_ask_pass_arguments_to_argparse(cls, parser: ArgumentParser):
        parser.add_argument('--ask-ssh-login', help='Ask for SSH username', action='store_true')
        parser.add_argument('--ask-ssh-pass', help='Ask for a SSH password', action='store_true')
        parser.add_argument('--ask-ssh-key-path', help='Ask for a SSH private key path', action='store_true')
        parser.add_argument('--ask-sudo-pass', help='Ask for sudo password', action='store_true')



import os
import subprocess
import time
import yaml
from dotenv import dotenv_values
from typing import Dict
from copy import deepcopy
from argparse import ArgumentParser
from rkd.api.contract import ExecutionContext
from rkd.api.inputoutput import BufferedSystemIO
from rkd.api.syntax import TaskDeclaration
from rkd.api.testing import FunctionalTestingCase
from .tasks.base import HarborBaseTask
from .service import ServiceDeclaration
from .driver import ComposeDriver
from .cached_loader import CachedLoader

HARBOR_MODULE_PATH = os.path.dirname(os.path.realpath(__file__))
ENV_SIMPLE_PATH = os.path.dirname(os.path.realpath(__file__)) + '/../../test/testdata/env_simple'
CURRENT_TEST_ENV_PATH = os.path.dirname(os.path.realpath(__file__)) + '/../../test/testdata/current_test_env'
TEST_PROJECT_NAME = 'env_simple'


class TestTask(HarborBaseTask):
    is_dev_env = True

    def get_name(self) -> str:
        return ':test'

    def get_group_name(self) -> str:
        return ''

    def execute(self, context: ExecutionContext) -> bool:
        return True

    def run(self, context: ExecutionContext) -> bool:
        return True

    def configure_argparse(self, parser: ArgumentParser):
        pass


class BaseHarborTestClass(FunctionalTestingCase):
    def setUp(self) -> None:
        super().setUp()

        print('')
        print('==================================================================================================' +
              '=====================================')
        print('Test name: ' + self.__class__.__name__ + ' :: ' + self._testMethodName)
        print('----------')
        print('')

        CachedLoader.clear()   # avoid keeping the state between tests

        os.chdir(HARBOR_MODULE_PATH)
        self.recreate_structure()
        self.setup_environment()
        self.remove_all_containers()

    @classmethod
    def mock_compose(cls, content: dict):
        content['version'] = '3.4'

        with open(CURRENT_TEST_ENV_PATH + '/apps/conf/mocked.yaml', 'wb') as f:
            f.write(yaml.dump(content).encode('utf-8'))

    @classmethod
    def recreate_structure(cls):
        """Within each class recreate the project structure, as it could be changed by tests itself"""

        subprocess.check_call(['rm', '-rf', CURRENT_TEST_ENV_PATH])
        subprocess.check_call(['cp', '-pr', ENV_SIMPLE_PATH, CURRENT_TEST_ENV_PATH])

        # copy from base structure - as we test eg. things like default configurations, NGINX template
        for directory in ['containers', 'data', 'hooks.d', 'apps/www-data']:
            subprocess.check_call('rm -rf %s/%s' % (ENV_SIMPLE_PATH, directory), shell=True)
            subprocess.check_call('cp -pr %s/project/%s %s/%s' % (
                HARBOR_MODULE_PATH, directory, CURRENT_TEST_ENV_PATH, directory
            ), shell=True)

        cls.mock_compose({'services': {}})

    @classmethod
    def get_test_env_subdirectory(cls, subdir_name: str):
        directory = CURRENT_TEST_ENV_PATH + '/' + subdir_name

        if not os.path.isdir(directory):
            subprocess.check_call(['mkdir', '-p', directory])

        return os.path.realpath(directory)

    @classmethod
    def remove_all_containers(cls):
        try:
            subprocess.check_output("docker rm -f -v $(docker ps -a --format '{{ .Names }}' | grep " + TEST_PROJECT_NAME + ")",
                                    shell=True, stderr=subprocess.STDOUT)

        except subprocess.CalledProcessError as e:
            # no containers found - it's OK
            if "requires at least 1 argument" in str(e.output):
                return

            raise e

    @classmethod
    def setup_environment(cls):
        os.environ.update(dotenv_values(CURRENT_TEST_ENV_PATH + '/.env'))
        os.environ['APPS_PATH'] = CURRENT_TEST_ENV_PATH + '/apps'
        os.environ['RKD_PATH'] = cls.get_test_env_subdirectory('') + ':' + HARBOR_MODULE_PATH + '/internal'

        os.chdir(CURRENT_TEST_ENV_PATH)

    def _get_prepared_compose_driver(self, args: dict = {}, env: dict = {}) -> ComposeDriver:
        merged_env = deepcopy(os.environ)
        merged_env.update(env)

        task = self.satisfy_task_dependencies(TestTask(), BufferedSystemIO())
        declaration = TaskDeclaration(task)
        ctx = ExecutionContext(declaration, args=args, env=merged_env)

        return ComposeDriver(task, ctx, TEST_PROJECT_NAME)

    @staticmethod
    def prepare_service_discovery(driver: ComposeDriver):
        driver.up(ServiceDeclaration('gateway', {}), capture=True, force_recreate=True)
        driver.up(ServiceDeclaration('gateway_proxy_gen', {}), capture=True, force_recreate=True)
        driver.up(ServiceDeclaration('website', {}), capture=True)

    def prepare_example_service(self, name: str, uses_service_discovery: bool = False) -> ComposeDriver:
        drv = self._get_prepared_compose_driver()

        # prepare
        drv.rm(ServiceDeclaration(name, {}))
        drv.up(ServiceDeclaration(name, {}))

        if uses_service_discovery:
            # give service discovery some time
            # @todo: This can be improved possibly
            time.sleep(10)

        return drv

    def get_containers_state(self, driver: ComposeDriver) -> Dict[str, bool]:
        running_rows = driver.scope.sh('docker ps -a --format "{{ .Names }}|{{ .Status }}"', capture=True).split("\n")
        containers = {}

        for container_row in running_rows:
            try:
                name, status = container_row.split('|')
            except ValueError:
                continue

            if name.startswith(driver.project_name + '_'):
                containers[name] = 'Up' in status

        return containers

    def get_locally_pulled_docker_images(self) -> list:
        images = subprocess.check_output(['docker', 'images', '--format', '{{ .Repository }}:{{ .Tag }}'])\
            .decode('utf-8')\
            .split("\n")

        return images

    def exec_in_container(self, container_name: str, cmd: list) -> str:
        return subprocess.check_output(
            ['docker', 'exec', '-i', container_name] + cmd,
            stderr=subprocess.STDOUT
        ).decode('utf-8')

    def fetch_page_content(self, host: str):
        return self.exec_in_container(TEST_PROJECT_NAME + '_gateway_1', ['curl', '-s', '-vv', '--header',
                                                                         'Host: %s' % host, 'http://127.0.0.1'])

    def prepare_valid_deployment_yml(self):
        """Internal: Prepares a valid deployment.yml and already synchronized files structure, downloaded role"""

        with open(self.get_test_env_subdirectory('') + '/deployment.yml', 'w') as f:
            f.write('''deploy_user: vagrant
deploy_group: vagrant
remote_dir: /project
git_url: https://github.com/riotkit-org/empty
git_secret_url: https://github.com/riotkit-org/empty
configure_sudoers: no

nodes:
    production:
        # example configuration for testing environment based on Vagrant
        # to run the environment type: harbor :deployment:vagrant -c "up --provision"
        - host: 127.0.0.1
          port: 2222
          user: docker
          password: docker
''')

    def prepare_valid_deployment_yml_encrypted_with_password_test(self):
        """Internal: Prepares a valid deployment.yml, same as prepare_valid_deployment_yml() but encrypted with vault

        Password is: test
        """

        with open(self.get_test_env_subdirectory('') + '/deployment.yml', 'w') as f:
            f.write('''$ANSIBLE_VAULT;1.1;AES256
39663866386139633138326634303139323266323236613732326366376531623239333465613438
3831373936306261316663383130316130643434343266630a643363303630626161333437313163
33313266316361653939373334323561326439323732363931303330313364363637303665303065
6262636637636633340a316436366131636363643233653163343733386632383439646336336137
33393138336364626265633438373661383835373463656465646361313832653337306332663434
35333034343865653361353261306464383336326130626266346336353738313066353639663539
61303031383738613333363034383166363332306665303533306465336335383234376230366339
38383134333431303333356666333535316465613865616539356538623734363434383164623865
32376438323665336564663135323163633561313730383930373339666166353761653038393930
35613233616562376337613232613861653562636132316238343434326530393338636465323031
35646532393135623561366634366561353132626532323732653534396435653833613934313030
30383763303161623733356134303863396365326330306437656465333761633833323662663564
63336435386566356232653263656162396132643264613261633235313237386465333362373537
30623131653235616366393337316638613863396364323831393430643363346366303631376632
39303361336365336266386639343432396533343639373832386533346263663965643765363738
39316235336539653962663232386436323034613464393732373432393033656366393139613834
35623230323733623235313935393838383161383866363165346235346230623932656631643838
64616661633039323166303934326163366532633732396162653236353965333135623264633939
62616532656637663061623230333838333432323134323761393334656339373338636530643237
34343634303138383138376364303236343532396638346562323764343665633165333433363937
34373665323733653435663938343665326162363862396139366438623062613963633565393166
39316431373561653163363035383365393430373833396631303563343030303836643533333631
61323534343963336265373465343438373536346235333633623130353536376461353233363636
35376361313332393236363235646431376630366164613764363332663639376234383963653231
33616236353832666262373037656332333637663962643462333662386363616266636635616266
39356436633130623561373061346338663062633636376438653764313539623831313433613764
64356532333466346264626362336632653335363461386436313536353334393865373565303839
34313962656236383736
''')

    def prepare_valid_deployment_yml_with_private_key(self):
        """
        Internal: Prepares a valid deployment.yml that has private key
        """

        with open(self.get_test_env_subdirectory('') + '/deployment.yml', 'w') as f:
            f.write('''deploy_user: vagrant
deploy_group: vagrant
remote_dir: /project
git_url: https://github.com/riotkit-org/empty
git_secret_url: https://github.com/riotkit-org/empty
configure_sudoers: no

nodes:
    production:
        # example configuration for testing environment based on Vagrant
        # to run the environment type: harbor :deployment:vagrant -c "up --provision"
        - host: 127.0.0.1
          port: 2222
          user: docker
          password: docker
          sudo_password: "sudo-docker"
          
          private_key: |
              -----BEGIN OPENSSH PRIVATE KEY-----
              b3BlbnNzaC1rZXktdjEAAAAABG5vbmUAAAAEbm9uZQAAAAAAAAABAAAAaAAAABNlY2RzYS
              1zaGEyLW5pc3RwMjU2AAAACG5pc3RwMjU2AAAAQQQO77Y+XtGXYUUb5VUVCy6EBX41aaTc
              hB4Qhz1dkpFDqZ86k+/acKOLyHxvN48nemEw0d7mhcQqq0yo2T1tqtfGAAAAsIQxjkeEMY
              5HAAAAE2VjZHNhLXNoYTItbmlzdHAyNTYAAAAIbmlzdHAyNTYAAABBBA7vtj5e0ZdhRRvl
              VRULLoQFfjVppNyEHhCHPV2SkUOpnzqT79pwo4vIfG83jyd6YTDR3uaFxCqrTKjZPW2q18
              YAAAAhANr1bWf4C6H8pZTxudh3Y1W2esX7ea1BFd8K8wdF43h3AAAAFGtyenlzaWVrQHJp
              b3RraXQtb3BzAQID
              -----END OPENSSH PRIVATE KEY-----
    ''')

    def assertContainerIsNotRunning(self, service_name: str, driver: ComposeDriver):
        container_name_without_instance_num = driver.project_name + '_' + service_name + '_'

        for name, state in self.get_containers_state(driver).items():
            if name.startswith(container_name_without_instance_num) and state is True:
                self.fail('"%s" is running, but should not' % name)

    def assertLocalRegistryHasImage(self, image_name):
        self.assertIn(image_name, self.get_locally_pulled_docker_images(),
                      msg='Expected that "docker images" will contain image "%s"' % image_name)

    def assertLocalRegistryHasNoPulledImage(self, image_name):
        self.assertNotIn(image_name, self.get_locally_pulled_docker_images(),
                         msg='Expected that "docker images" will not contain image "%s"' % image_name)
